@@ -5,7 +5,7 @@ import pandas as pd
 
 from pandas.api.types import is_categorical_dtype, is_numeric_dtype, is_string_dtype
 
-from formulae.transforms import TRANSFORMS, Prop, Offset
+from formulae.transforms import TRANSFORMS, Proportion, Offset
 from formulae.terms.call_utils import CallVarsExtractor
 
 
@@ -31,7 +31,7 @@ class Call:
 
     def __init__(self, call, is_response=False):
         self.data = None
-        self.eval_env = None
+        self.env = None
         self._intermediate_data = None
         self._type = None
         self.is_response = is_response
@@ -78,7 +78,7 @@ class Call:
         """
         return set(CallVarsExtractor(self).get())
 
-    def set_type(self, data_mask, eval_env):
+    def set_type(self, data_mask, env):
         """Evaluates function and determines the type of the result of the call.
 
         Evaluates the function call and sets the ``._type`` property to ``"numeric"`` or
@@ -90,21 +90,22 @@ class Call:
         ----------
         data_mask: pd.DataFrame
             The data frame where variables are taken from
-        eval_env: EvalEnvironment
+        env: Environment
             The environment where values and functions are taken from.
         """
 
-        self.eval_env = eval_env.with_outer_namespace(TRANSFORMS)
-        x = self.call.eval(data_mask, self.eval_env)
+        self.env = env.with_outer_namespace(TRANSFORMS)
+        x = self.call.eval(data_mask, self.env)
 
         if is_numeric_dtype(x):
             self._type = "numeric"
         elif is_string_dtype(x) or is_categorical_dtype(x):
             self._type = "categoric"
-        elif isinstance(x, Prop):
-            self._type = "prop"
+        elif isinstance(x, Proportion):
+            self._type = "proportion"
         elif isinstance(x, Offset):
             self._type = "offset"
+            x.set_size(len(data_mask.index))
         else:
             raise ValueError(f"Call result is of an unrecognized type ({type(x)}).")
         self._intermediate_data = x
@@ -127,14 +128,14 @@ class Call:
         try:
             if self._type is None:
                 raise ValueError("Call result type is not set.")
-            if self._type not in ["numeric", "categoric", "prop", "offset"]:
+            if self._type not in ["numeric", "categoric", "proportion", "offset"]:
                 raise ValueError(f"Call result is of an unrecognized type ({self._type}).")
             if self._type == "numeric":
                 self.data = self._eval_numeric(self._intermediate_data)
             elif self._type == "categoric":
                 self.data = self._eval_categoric(self._intermediate_data, encoding)
-            elif self._type == "prop":
-                self.data = self._eval_prop(self._intermediate_data)
+            elif self._type == "proportion":
+                self.data = self._eval_proportion(self._intermediate_data)
             elif self._type == "offset":
                 self.data = self._eval_offset(self._intermediate_data)
         except:
@@ -160,10 +161,10 @@ class Call:
             evaluation, and the latter is equal to ``"numeric"``.
         """
         if isinstance(x, np.ndarray):
-            value = x.flatten()
-            if value.ndim > 1:
-                raise ValueError(f"The result of {self.name} is not 1-dimensional.")
-            value = value[:, np.newaxis]
+            if x.ndim == 1:
+                value = x[:, np.newaxis]
+            else:
+                value = x
         elif isinstance(x, pd.Series):
             value = x.to_numpy()[:, np.newaxis]
         else:
@@ -228,17 +229,17 @@ class Call:
             "encoding": encoding,
         }
 
-    def _eval_prop(self, prop):
+    def _eval_proportion(self, proportion):
         if not self.is_response:
             raise ValueError("'prop()' can only be used in the context of a response term.")
-        return {"value": prop.eval(), "type": "prop"}
+        return {"value": proportion.eval(), "type": "proportion"}
 
     def _eval_offset(self, offset):
         if self.is_response:
             raise ValueError("'offset() cannot be used in the context of a response term.")
         return {"value": offset.eval(), "type": "offset"}
 
-    def eval_new_data(self, data_mask):
+    def eval_new_data(self, data_mask):  # pylint: disable = inconsistent-return-statements
         """Evaluates the function call with new data.
 
         This method evaluates the function call within a new data mask. If the transformation
@@ -257,11 +258,34 @@ class Call:
             ``self._eval_categoric()``. The first applies for numeric calls, the second for
             categoric ones.
         """
-        x = self.call.eval(data_mask, self.eval_env)
-        if self._type == "numeric":
-            return self._eval_numeric(x)["value"]
-        else:
-            return self._eval_new_data_categoric(x)
+        if self._type in ["numeric", "categoric"]:
+            x = self.call.eval(data_mask, self.env)
+            if self._type == "numeric":
+                return self._eval_numeric(x)["value"]
+            else:
+                return self._eval_new_data_categoric(x)
+        elif self._type == "proportion":
+            if self._intermediate_data.trials_type == "constant":
+                # Return value passed in the second component
+                return np.ones((len(data_mask.index), 1)) * self.call.args[1].value
+            else:
+                # Extract name of the second component
+                name = self.call.args[1].name
+                values = data_mask[name]
+                if isinstance(values, pd.Series):
+                    values = values.values[:, np.newaxis]
+                return values
+        elif self._type == "offset":
+            if self._intermediate_data.type == "constant":
+                # Return value passed as the argument
+                return np.ones((len(data_mask.index), 1)) * self.call.args[0].value
+            else:
+                # Extract name of the argument
+                name = self.call.args[0].name
+                values = data_mask[name]
+                if isinstance(values, pd.Series):
+                    values = values.values[:, np.newaxis]
+                return values
 
     def _eval_new_data_categoric(self, x):
         """Evaluates the call with new data when the result of the call is categoric.
@@ -279,6 +303,7 @@ class Call:
             number of dummy variables used in the numeric representation of the categorical
             variable.
         """
+
         # Raise error if passing a level that was not observed.
         new_data_levels = pd.Categorical(x).dtype.categories.tolist()
         if set(new_data_levels).issubset(set(self.data["levels"])):
