@@ -1,10 +1,11 @@
-from numpy.core.numeric import allclose
 import pytest
+import re
 
 import numpy as np
 import pandas as pd
 
 from formulae.matrices import design_matrices
+from formulae.parser import ParseError
 
 # TODO: See interaction names.. they don't always work as expected
 @pytest.fixture(scope="module")
@@ -536,8 +537,25 @@ def test_prop_response():
     )
 
     response = design_matrices("prop(y, n) ~ x", data).response
+    assert response.type == "proportion"
+    assert response.design_vector.shape == (8, 2)
+    assert (np.less_equal(response.design_vector[:, 0], response.design_vector[:, 1])).all()
 
-    assert response.type == "prop"
+    # Admit integer values for 'n'
+    response = design_matrices("prop(y, 62) ~ x", data).response
+    assert response.type == "proportion"
+    assert response.design_vector.shape == (8, 2)
+    assert (np.less_equal(response.design_vector[:, 0], response.design_vector[:, 1])).all()
+
+    # Use aliases
+    response = design_matrices("proportion(y, n) ~ x", data).response
+    assert response.type == "proportion"
+    assert response.design_vector.shape == (8, 2)
+    assert (np.less_equal(response.design_vector[:, 0], response.design_vector[:, 1])).all()
+
+    # Use aliases
+    response = design_matrices("p(y, n) ~ x", data).response
+    assert response.type == "proportion"
     assert response.design_vector.shape == (8, 2)
     assert (np.less_equal(response.design_vector[:, 0], response.design_vector[:, 1])).all()
 
@@ -554,12 +572,21 @@ def test_prop_response_fails():
     with pytest.raises(ValueError):
         design_matrices("prop(x, n) ~ 1", pd.DataFrame({"x": [2, 3], "n": [4.3, 4]}))
 
+    # x not a variable name
+    with pytest.raises(ValueError):
+        design_matrices("prop(10, n) ~ 1", pd.DataFrame({"x": [2, 3], "n": [1, 2]}))
+
+    # trials must be integer, not float
+    with pytest.raises(ValueError):
+        design_matrices("prop(x, 3.4) ~ 1", pd.DataFrame({"x": [2, 3], "n": [1, 2]}))
+
 
 def test_categoric_responses():
     data = pd.DataFrame(
         {
             "y1": np.random.choice(["A", "B", "C"], size=30),
             "y2": np.random.choice(["A", "B"], size=30),
+            "y3": np.random.choice(["Hi there", "Bye bye", "What??"], size=30),
             "x": np.random.normal(size=30),
         }
     )
@@ -596,6 +623,32 @@ def test_categoric_responses():
     assert response.baseline is None
     assert response.success == "B"
 
+    # Binary response with explicit level passed as identifier
+    response = design_matrices("y2[B] ~ x", data).response
+    assert list(np.unique(response.design_vector)) == [0, 1]
+    assert response.levels == ["A", "B"]
+    assert response.binary is True
+    assert response.baseline is None
+    assert response.success == "B"
+
+    # Binary response with explicit level with spaces
+    response = design_matrices("y3['Bye bye'] ~ x", data).response
+    assert list(np.unique(response.design_vector)) == [0, 1]
+    assert response.levels == ["Bye bye", "Hi there", "What??"]
+    assert response.binary is True
+    assert response.baseline is None
+    assert response.success == "Bye bye"
+
+    # Users trying to use nested brackets (WHY?)
+    with pytest.raises(ParseError, match=re.escape("Are you using nested brackets? Why?")):
+        design_matrices("y3[A[B]] ~ x", data)
+
+    # Users try to pass a number to use a number
+    with pytest.raises(
+        ParseError, match=re.escape("Subset notation only allows a string or an identifer")
+    ):
+        design_matrices("y3[1] ~ x", data)
+
 
 def test_binary_function():
     size = 100
@@ -705,3 +758,43 @@ def test_offset():
 
     with pytest.raises(ValueError):
         design_matrices("offset(y) ~ x", data)
+
+
+def test_predict_prop():
+    data = pd.DataFrame(
+        {
+            "x": np.array([1.6907, 1.7242, 1.7552, 1.7842, 1.8113, 1.8369, 1.8610, 1.8839]),
+            "n": np.array([59, 60, 62, 56, 63, 59, 62, 60]),
+            "y": np.array([6, 13, 18, 28, 52, 53, 61, 60]),
+        }
+    )
+
+    # If trials is a variable, new dataset must have that variable
+    dm = design_matrices("prop(y, n) ~ x", data)
+    result = dm.response._evaluate_new_data(pd.DataFrame({"n": [10, 10, 30, 30]}))
+    assert (result == np.array([10, 10, 30, 30])[:, np.newaxis]).all()
+
+    # If trials is a constant value, return that same value
+    dm = design_matrices("prop(y, 70) ~ x", data)
+    result = dm.response._evaluate_new_data(pd.DataFrame({"n": [10, 10, 30, 30]}))
+    assert (result == np.array([70, 70, 70, 70])[:, np.newaxis]).all()
+
+
+def test_predict_offset():
+    data = pd.DataFrame(
+        {
+            "x": np.array([1.6907, 1.7242, 1.7552, 1.7842, 1.8113, 1.8369, 1.8610, 1.8839]),
+            "n": np.array([59, 60, 62, 56, 63, 59, 62, 60]),
+            "y": np.array([6, 13, 18, 28, 52, 53, 61, 60]),
+        }
+    )
+
+    # If offset is a variable, new dataset must have that variable
+    dm = design_matrices("y ~ x + offset(x)", data)
+    result = dm.common._evaluate_new_data(pd.DataFrame({"x": [1, 2, 3]}))["offset(x)"]
+    assert (result == np.array([1, 2, 3])[:, np.newaxis]).all()
+
+    # If offset is a constant value, return that same value
+    dm = design_matrices("y ~ x + offset(10)", data)
+    result = dm.common._evaluate_new_data(pd.DataFrame({"x": [1, 2, 3]}))["offset(10)"]
+    assert (result == np.array([10, 10, 10])[:, np.newaxis]).all()
